package system;

import data_access.DataAccess;
import model.Article;
import model.Customer;
import model.Order;
import model.OrderItem;


import java.awt.*;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

class LogicImpl implements Logic{
    private final List<Customer> customers;
    private final List<Article> articles;
    private final List<Order> orders;
    private DataAccess dataAccess;




    public LogicImpl(DataAccess dataAccess) {
        this.customers = new ArrayList<Customer>();
        this.articles = new ArrayList<Article>();
        this.orders = new ArrayList<Order>();
    }

    //Berechnung der MwST.
/*
    private double vat( long brutto, double vatRate ){
        //berechnet man die Umsatzsteuer am besten in zwei Schritten:
        // man dividiert den Bruttobetrag durch 1,19 und
        // erhält damit den Nettobetrag, diesen multipliziert man wieder mit 19%.
        double total = ((double)brutto / 100);
        double vatRateCalc = 1.19 ;
        double vatrate = ((double)total / vatRateCalc) * (vatRate/100);
        //return vatrate;

       double erg = Math.round(vatrate * 100) / 100D;
       return (long)erg;
    }*/

    private long vat( long brutto, long vatRate ) {
        double bruttoCalc = ((double)brutto/(100+(vatRate)));
        double vatCalc = (bruttoCalc*(vatRate));
        return Math.round(vatCalc);
    }


    private String fmtPrice( long price, String currency ) {
        double total = ( (double)price ) / 100.0; // convert cent to Euro
        DecimalFormat df = new DecimalFormat( "#,##0.00", new DecimalFormatSymbols(
                new Locale( "de" ) ) );
        // round to 2 digits
        String fmtPrice = df.format( total );
        fmtPrice += ( currency != null && currency.length() > 0 )? " " + currency : "";
        return fmtPrice;
    }


    @Override
    public void printOrders(Iterable<Order> orders) {
        long endSum = 0;
        long resultvat = 0;
        double sumVAT = 0.0;

        //Ausdruck einer Liste von Bestellungen
        for(Order order : orders){
            long id = order.getId();
            String name = order.getCustomer().getFirstName();
            StringBuffer sb = new StringBuffer();
            StringBuffer ab = new StringBuffer();
            StringBuffer bb = new StringBuffer();
            sb.append( name ).append( "'s Bestellung #(" ).append( id ).append( "): " );

            for( int i = 0; i < order.getItems().size(); i++ ) {
                OrderItem item = order.getItems().get(i);
                sb.append(i > 0 ? ", " : "")
                        .append(item.getUnitsOrdered()).append("x ")
                        .append(item.getDescription());
            }

            System.out.println( sb.append( "." ).toString() );

            for( int j = 0; j < order.getItems().size(); j++){
               OrderItem item = order.getItems().get(j);
                ab.append(j > 0 ? "; " : "").append(" ==> ")
                        .append(item.getUnitsOrdered()).append("x ")
                        .append(fmtPrice(item.getArticle().getUnitPrice(), "EUR"));

                bb.append(j > 0 ? "" : " ");

            }
            bb.append(this.fmtPrice(calculateTotal(order),"EUR ("));
            bb.append(this.vat(calculateTotal(order),19));
            bb.append(" MWSt)");

            long sum = calculateTotal(order);
            endSum += sum;

            System.out.println( ab.append ( "; ").toString() + "==> Total:" + bb.append(""));
        }

        System.out.println();
        System.out.println(" == ==> Total (all orders): " + fmtPrice(endSum, "EUR") + " (" + vat(endSum, 19) + " MWSt)");
        System.out.println("");


        this.doInventory((List<Order>) orders);

    }

    public void doInventory(List<Order> orders) {
        for(Order order : orders){
            for(OrderItem item : order.getItems()) {
                int unitsOrdered = item.getUnitsOrdered();
                String id = item.getArticle().getId();


                for(Article article : this.articles) {
                    if (article.getId().equals(id)) {
                        int currentUnits = article.getUnitsInStore();
                        article.setUnitsInStore(currentUnits - unitsOrdered);

                    }
                }
            }
        }
    }

    @Override
    public void printInventory() {

        System.out.println("Available Inventory:");
        long endSum = 0;
        for(Article article : this.articles){
            StringBuffer sb = new StringBuffer();
            long units = article.getUnitsInStore();
            long sum = article.getUnitPrice() * article.getUnitsInStore();
            DecimalFormat df = new DecimalFormat( "##,000", new DecimalFormatSymbols(
                    new Locale( "de" ) ) );
            String fmtUnits = df.format(units);

            sb.append(article.getId() + ", ");



            if(article.getDescription().length() > 13 ) {
                sb.append((article.getDescription() + "              ").substring(0, 14).trim() + "\t\t\t-->\t");

            }else if(article.getDescription().length() > 8){
                sb.append((article.getDescription() + "               ").substring(0, 14).trim() + "\t\t\t\t-->\t");

            } else {
                sb.append((article.getDescription() + "               ").substring(0,14).trim() + "\t\t\t\t\t-->\t");

            }

           if(article.getDescription().length() > 13){
                sb.append(fmtUnits + "\t\tunits x\t");

            } else {
                sb.append(fmtUnits + "\tunits x\t");
            }



           // sb.append(article.getDescription() + " \t\t");
           // sb.append(fmtUnits + "\tunits x\t\t");
            sb.append(fmtPrice(article.getUnitPrice(), ",") + "\ttotal value: \t");
            sb.append(fmtPrice(sum,"EUR"));

            endSum+= sum;
            System.out.println(sb.toString());


        }
        System.out.println("---------------------------------------------------------------------------------------------");
        System.out.println("\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t " + "  " + fmtPrice(endSum,"EUR"));

    }
/*
    //Berechnung des Gesamtwerts einer Bestellung, d.h. die
    //Summe über die einzelnen Bestellpositionen
    @Override
    public long calculateTotal(Order order) {
        long total = 0;
        long current_price = 0;
        List<OrderItem> items = order.getItems();
        for(OrderItem i : items){
           long article = i.getArticle().getUnitPrice();
           long count = i.getUnitsOrdered();
           current_price = article * count;
           total += current_price;

        }
        return total;
    }*/

    @Override
    public long calculateTotal(Order order) {
        long orderTotal = 0L;
        for( OrderItem item : order.getItems() ) {
            orderTotal += (item.getUnitsOrdered() * item.getArticle().getUnitPrice());
        }
        return orderTotal;
    }

    //Berechnung der im Endpreis einer Bestellung
    //enthaltenen Mehrwertsteuer (engl. Value‐Added Tax VAT) zum Satz von 19%.
    @Override
    public long calculateIncludedVAT(Order order) {
        long erg = vat(calculateTotal(order),19);
         //erg += (calculateTotal(order) / 100L);
        //System.out.println("VAT:" + erg);
        long ergL = erg;
        return ergL;
    }

    @Override
    public boolean fillOrder(Order order) {
        for(OrderItem i : order.getItems()) {
            // Anzahl bestellt von order
            int units = i.getUnitsOrdered();

            // Id des bestellten Artikel:
            String id = i.getArticle().getId();

            // Über Bestand iterieren:
            for(Article article : this.articles) {
                // Suchst die id  -> anzahl vergleichen: article.anzahl mit units
                // units < anzahl -> true -> inventur
                if(id.equals(article.getId())){
                    // wenn mehr im lager ist als bestellt:
                    if(article.getUnitsInStore() >= units){
                        return true;

                        // unit > anzahl -> false
                    }else{
                        return false;
                    }
                }
            }
        }
        return false;
    }


    @Override
    public void startup() {

        Customer cEric = new Customer( "C86516", "Eric Schulz-Mueller", "eric2346@gmail.com" );
        Customer cAnne = new Customer( "C64327", "Meyer, Anne", "+4917223524" );
        Customer cNadine = new Customer( "C12396", "Nadine Ulla Blumenfeld", "+4915292454" );
        Customer cTimo = new Customer( "C16865", "Timo Werner", "tw@gmail.com" );
        Customer cSandra = new Customer("C66865","Sandra Mueller","samue62@gmx.de" );


        this.customers.add( cEric );
        this.customers.add( cAnne );
        this.customers.add( cNadine );
        this.customers.add( cTimo );
        this.customers.add( cSandra );

        /////////////////////////////////////

        Article a1 = new Article( "SKU-458362", "Tasse", 299, 2000 );
        Article a2 = new Article( "SKU-693856", "Becher", 149, 8400 );
        Article a3 = new Article( "SKU-518957", "Kanne", 2000, 2400 );
        Article a4 = new Article( "SKU-638035", "Teller", 649, 7000 );
        Article a5 = new Article( "SKU-386035", "Kaffeemaschine", 2999,500);
        Article a6 = new Article( "SKU-443803", "Teekocher", 1999, 2000);

        this.articles.add(a1);
        this.articles.add(a2);
        this.articles.add(a3);
        this.articles.add(a4);
        this.articles.add(a5);
        this.articles.add(a6);





        // Eric's 1st order
        Order o5234 = new Order( 5234968294L, new Date(), cEric );
        OrderItem oi1 = new OrderItem( a3.getDescription(), a3, 1 );	// 1 Kanne (a3)
        o5234.addItem( oi1 );	// add OrderItem to Order 5234968294L

        // Eric's 2nd order
        Order o8592 = new Order( 8592356245L, new Date(), cEric );
        o8592.addItem(	// add three OrderItems to Order 8592356245L
                new OrderItem( a4.getDescription(), a4, 4 )		// 4 Teller (a4)
        ).addItem(
                new OrderItem( a2.getDescription(), a2, 8 )		// 8 Becher (a2)
        ).addItem(
                new OrderItem( "passende Tassen", a1, 4 )		// 4 passende Tassen (a1)
        );

        // Anne's order
        Order o3563 = new Order( 3563561357L, new Date(), cAnne );
        o3563.addItem(
                new OrderItem( a3.getDescription() + " aus Porzellan", a3, 1 )
        );

        // Nadine's order
        Order o6135 = new Order( 6135735635L, new Date(), cNadine );
        o6135.addItem(	// 12 Teller
                new OrderItem( a4.getDescription() + " blau/weiss Keramik", a4, 12 )
        );


        //Timo's order
        Order o1234 = new Order(9235856245L, new Date(), cTimo);
        o1234.addItem(
                new OrderItem(a5.getDescription() , a5,1) //1 Kaffeemaschine
        ).addItem(
                new OrderItem(a1.getDescription(), a1,6 )); // 6 Tassen

        //Sandra's order
        Order o5431 = new Order(7335856245L, new Date(), cSandra);
        o5431.addItem(
                new OrderItem(a6.getDescription() , a6,1 ) // 1 Teekocher
        ).addItem(
                new OrderItem(a2.getDescription(), a2,4 ) // 4 Becher
        ).addItem(
                new OrderItem(a4.getDescription(), a4,4 ) // 4 teller
        );


        this.orders.add(o5234);
        this.orders.add(o8592);
        this.orders.add(o3563);
        this.orders.add(o6135);
        this.orders.add(o1234);
        this.orders.add(o5431);

    }

    @Override
    public void shutdown() {

        orders.removeAll(orders);
        articles.removeAll(articles);
        customers.removeAll(customers);

    }
}
